`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2019/04/02 16:30:11
// Design Name: 
// Module Name: topmodule
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////

module topmodule(
    D_ssd,
    ssd_ctl,
    clk,
    rst_n,
    pb_do,
    pb_re,
    pb_mi,
    volume_up,
    volume_down,
    audio_lrck,
    audio_mclk,
    audio_sck,
    audio_sdin
    );

    output [7:0] D_ssd;
    output [3:0] ssd_ctl;
    output audio_lrck, audio_mclk, audio_sck, audio_sdin;
    input clk;
    input rst_n;
    input volume_up, volume_down, pb_do, pb_re, pb_mi;
    
    reg [21:0] note_div_target;

    wire clk_1Hz;
    wire clk_100Hz;
    wire [1:0] clk_ssd_ctl;

    wire debounced_pb_do, debounced_pb_re, debounced_pb_mi;
    wire debounced_volume_down, debounced_volume_up;

    wire out_pulse_volume_down, out_pulse_volume_up;

    wire [3:0] volume_level;
    wire [15:0] audio_in_left, audio_in_right;

    wire [3:0] ssd_in;
   

    clock_generator U0(
    .clk_1Hz(clk_1Hz), //divided clock output
    .clk_100Hz(clk_100Hz),
    .clk_ctl(clk_ssd_ctl), //clock for 7-segment
    .clk(clk),    //clock input
    .rst_n(rst_n)   //low-active reset
    );

    debounce U1( 
    .rst_n(rst_n), 
    .clk(clk_100Hz), 
    .pb_in(pb_do),
    .pb_debounced(debounced_pb_do)
    );

    debounce U2( 
    .rst_n(rst_n), 
    .clk(clk_100Hz), 
    .pb_in(pb_re),
    .pb_debounced(debounced_pb_re)
    );

    debounce U3( 
    .rst_n(rst_n), 
    .clk(clk_100Hz), 
    .pb_in(pb_mi),
    .pb_debounced(debounced_pb_mi)
    );

    debounce U4( 
    .rst_n(rst_n), 
    .clk(clk_100Hz), 
    .pb_in(volume_up),
    .pb_debounced(debounced_volume_up)
    );

    debounce U5( 
    .rst_n(rst_n), 
    .clk(clk_100Hz), 
    .pb_in(volume_down),
    .pb_debounced(debounced_volume_down)
    );

    one_pulse U6( 
    .rst_n(rst_n), 
    .clk(clk), 
    .in_trig(debounced_volume_down),
    .out_pulse(out_pulse_volume_down)
    );

    one_pulse U7( 
    .rst_n(rst_n), 
    .clk(clk), 
    .in_trig(debounced_volume_up),
    .out_pulse(out_pulse_volume_up)
    );
    

    always @*
        begin
            if (debounced_pb_do == 1'b1)
                note_div_target = 22'd191571;
            else if (debounced_pb_re == 1'b1)
                note_div_target = 22'd170648;
            else if (debounced_pb_mi == 1'b1)
                note_div_target = 22'd151515;
            else
                note_div_target = 22'd0;
        end

    volume_ctl U77( 
    .rst_n(rst_n), 
    .clk(clk), 
    .volume_up(out_pulse_volume_up),
    .volume_down(out_pulse_volume_down),
    .curr_state(volume_level)
);
    note_gen U8(
    .clk(clk),
    .rst_n(rst_n),
    .note_div(note_div_target),
    .audio_left(audio_in_left),
    .audio_right(audio_in_right),
    .level_change(volume_level)
    );

    speaker_control U9(
    .clk(clk),
    .rst_n(rst_n),
    .audio_in_left(audio_in_left),
    .audio_in_right(audio_in_right),
    .audio_mclk(audio_mclk),
    .audio_lrck(audio_lrck),
    .audio_sck(audio_sck),
    .audio_sdin(audio_sdin)
    );


    scan_freq_ctl U10(
    .ssd_in(ssd_in),
    .ssd_ctl(ssd_ctl),
    .clk_ctl_en(clk_ssd_ctl),
    .rst_n(rst_n),
    .volume_level(volume_level)
    );
    
    display U11(.D_ssd(D_ssd), .i(ssd_in));

endmodule
